@@ -1,14 +1,24 @@
 package Net::DNS;
 #
-# $Id: DNS.pm 114 2004-08-13 00:23:41Z ctriv $
+# $Id: DNS.pm 468 2005-07-22 12:12:55Z olaf $
 #
 use strict;
+
+
+BEGIN { 
+    eval { require bytes; }
+}
+
+
+
 use vars qw(
     $HAVE_XS
     $VERSION
     $DNSSEC
+    $DN_EXPAND_ESCAPES
     @ISA
     @EXPORT
+    @EXPORT_OK
     %typesbyname
     %typesbyval
     %qtypesbyname
@@ -28,26 +38,41 @@ use vars qw(
 BEGIN {
     require DynaLoader;
     require Exporter;
-    
     @ISA     = qw(Exporter DynaLoader);
-    $VERSION = '0.48';
-    $HAVE_XS = eval { __PACKAGE__->bootstrap(); 1 } ? 1 : 0;
+
+    
+    $VERSION = '0.53';
+    $HAVE_XS = eval { 
+	local $SIG{'__DIE__'} = 'DEFAULT';
+	__PACKAGE__->bootstrap(); 1 
+	} ? 1 : 0;
+
 }
 
+
+
+BEGIN {
+
+    $DNSSEC = eval { 
+	    local $SIG{'__DIE__'} = 'DEFAULT';
+	    require Net::DNS::SEC; 
+	    1 
+	    } ? 1 : 0;
+
+
+}
+
+
 use Net::DNS::Resolver;
 use Net::DNS::Packet;
 use Net::DNS::Update;
 use Net::DNS::Header;
 use Net::DNS::Question;
-use Net::DNS::RR;
-
-BEGIN {
-    $DNSSEC = eval { require Net::DNS::RR::SIG; 1 } ? 1 : 0;
-}
- 
+use Net::DNS::RR;   # use only after $Net::DNS::DNSSEC has been evaluated
 
 
 @EXPORT = qw(mx yxrrset nxrrset yxdomain nxdomain rr_add rr_del);
+@EXPORT_OK= qw(name2labels wire2presentation );
 
 #
 # If you implement an RR record make sure you also add it to 
@@ -90,7 +115,7 @@ BEGIN {
     'AAAA'      => 28,      # RFC 1886, Section 2.1
     'LOC'       => 29,      # RFC 1876
     # The following RR is impemented in Net::DNS::SEC
-    'NXT'       => 30,      # RFC 2535, Section 5.2
+    'NXT'       => 30,      # RFC 2535, Section 5.2 obsoleted by RFC3755
     'EID'       => 31,      # draft-ietf-nimrod-dns-xx.txt
     'NIMLOC'    => 32,      # draft-ietf-nimrod-dns-xx.txt
     'SRV'       => 33,      # RFC 2052
@@ -101,12 +126,11 @@ BEGIN {
     'DNAME'     => 39,      # RFC 2672
     'OPT'       => 41,      # RFC 2671
     # The following 4 RRs are impemented in Net::DNS::SEC
-    # Aug 2003: These RRs will be published as RFCs shortly 
-    'DS'        => 43,      # draft-ietf-dnsext-delegation-signer
-    'SSHFP'     => 44,          # draft-ietf-secsh-dns (No RFC # yet at time of coding)
-    'RRSIG'     => 46,      # draft-ietf-dnsext-dnssec-2535typecode-change
-    'NSEC'      => 47,      # draft-ietf-dnsext-dnssec-2535typecode-change
-    'DNSKEY'    => 48,      # draft-ietf-dnsext-dnssec-2535typecode-change
+    'DS'        => 43,      # RFC 4034
+    'SSHFP'     => 44,      # draft-ietf-secsh-dns (No RFC # yet at time of coding)
+    'RRSIG'     => 46,      # RFC 4034
+    'NSEC'      => 47,      # RFC 4034
+    'DNSKEY'    => 48,      # RFC 4034
     'UINFO'     => 100,     # non-standard
     'UID'       => 101,     # non-standard
     'GID'       => 102,     # non-standard
@@ -157,10 +181,9 @@ sub typesbyval {
     my $val = shift;
     
     die "Net::DNS::typesbyval() argument ($val) is not numeric" unless 
-        $val =~ m/^\s*\d+\s*$/;  
+	$val =~ m/^\s*0*(\d+)\s*$/;
+    $val = $1;
     
-    $val =~ s/\s*//g; 
-    $val =~ s/^0*//; 
     
     return $typesbyval{$val} if $typesbyval{$val};
     
@@ -216,7 +239,7 @@ sub classesbyval {
         $val=~/^\s*\d+\s*$/;
     
     $val =~ s/\s*//g;
-    $val =~ s/^0*//;
+    $val =~ s/^0*([0-9]+)/$1/;    #remove leading zeros 
     
     return $classesbyval{$val} if $classesbyval{$val};
     
@@ -337,6 +360,131 @@ sub rr_del {
     return Net::DNS::RR->new_from_string(shift, 'rr_del');
 }
 
+
+
+# Utility function
+#
+# name2labels to translate names from presentation format into an
+# array of "wire-format" labels.
+
+
+# in: $dname a string with a domain name in presentation format (1035
+# sect 5.1)
+# out: an array of labels in wire format.
+
+
+sub name2labels {
+    my $dname=shift;
+    my @names;
+    my $j=0;
+    while ($dname){
+	($names[$j],$dname)=presentation2wire($dname);
+	$j++;
+    }
+
+    return @names;
+}
+
+
+
+
+sub wire2presentation {
+    my  $wire=shift;
+    my  $presentation="";
+    my $length=length($wire);
+    # There must be a nice regexp to do this.. but since I failed to
+    # find one I scan the name string until I find a '\', at that time
+    # I start looking forward and do the magic.
+
+    my $i=0;
+    
+    while ($i < $length ){
+	my $char=unpack("x".$i."C1",$wire);
+	if ( $char < 33 || $char > 126 ){
+	    $presentation.= sprintf ("\\%03u" ,$char);
+	}elsif ( $char == ord( "\"" )) {   
+	    $presentation.= "\\\"";    
+	}elsif ( $char == ord( "\$" )) {   
+	    $presentation.= "\\\$";    
+	}elsif ( $char == ord( "(" )) {   
+	    $presentation.= "\\(";    
+	}elsif ( $char == ord( ")" )) {   
+	    $presentation.= "\\)";    
+	}elsif ( $char == ord( ";" )) {   
+	    $presentation.= "\\;";    
+	}elsif ( $char == ord( "@" )) {   
+	    $presentation.= "\\@";    
+	}elsif ( $char == ord( "\\" )) {   
+	    $presentation.= "\\\\" ; 
+	}elsif ( $char==ord (".") ){
+	    $presentation.= "\\." ; 
+	}else{
+	    $presentation.=chr($char) 	;
+	}
+	$i++;
+    }
+
+    return $presentation;
+    
+}
+
+
+
+# ($wire,$leftover)=presentation2wire($leftover);
+
+# Will parse the input presentation format and return everything before
+# the first non-escaped "." in the first element of the return array and
+# all that has not been parsed yet in the 2nd argument.
+
+
+sub presentation2wire {
+    my  $presentation=shift;
+    my  $wire="";
+    my $length=length($presentation);
+    
+    my $i=0;
+    
+    while ($i < $length ){
+	my $char=unpack("x".$i."C1",$presentation);
+	if (  $char == ord ('.')){
+	    return ($wire,substr($presentation,$i+1));
+	}
+	if (  $char == ord ('\\')){
+	    #backslash found
+	    pos($presentation)=$i+1;
+	    if ($presentation=~/\G(\d\d\d)/){
+		$wire.=pack("C",$1);
+		$i+=3;
+	    }elsif($presentation=~/\Gx([0..9a..fA..F][0..9a..fA..F])/){
+		$wire.=pack("H*",$1);
+		$i+=3;
+	    }elsif($presentation=~/\G\./){
+		$wire.="\.";
+		$i+=1;
+	    }elsif($presentation=~/\G@/){
+		$wire.="@";
+		$i+=1;
+	    }elsif($presentation=~/\G\(/){
+		$wire.="(";
+		$i+=1;
+	    }elsif($presentation=~/\G\)/){
+		$wire.=")";
+		$i+=1;
+           }elsif($presentation=~/\G\\/){
+               $wire.="\\"; 
+               $i+=1;
+	    }
+	}else{
+	    $wire .=  pack("C",$char);  
+        }
+	$i++;
+    }
+    
+    return $wire;
+}
+
+
+
 1;
 __END__
 
@@ -551,6 +699,7 @@ section of a dynamic update packet.
 Returns a C<Net::DNS::RR> object or C<undef> if the object couldn't
 be created.
 
+
 =head1 EXAMPLES
 
 The following examples show how to use the C<Net::DNS> modules.
@@ -687,22 +836,33 @@ Copyright (c) 1997-2002 Michael Fuhr.
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
+Portions Copyright (c) 2005 Olaf Kolkman (RIPE NCC)
+
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
 
 =head1 AUTHOR INFORMATION
 
-Net::DNS is currently maintained by a group, led by:
-    Chris Reinhardt
-    ctriv@net-dns.org
+Net::DNS is currently maintained by:
+        Olaf Kolkman
+	olaf@net-dns.org
+
+Between 2002 and 2004 Net::DNS was maintained by:
+       Chris Reinhardt
+
 
 Net::DNS was created by:
-    Michael Fuhr
-    mike@fuhr.org
+	Michael Fuhr
+	mike@fuhr.org 
+
+
 
 For more information see:
     http://www.net-dns.org/
 
+Stay tuned and syncicate:
+    http://www.net-dns.org/blog/
+
 =head1 SEE ALSO
  
 L<perl(1)>, L<Net::DNS::Resolver>, L<Net::DNS::Packet>, L<Net::DNS::Update>,
